@@ -35,6 +35,7 @@
 #include "../data-plane/data-plane.h"
 #include "../lib/oor_log.h"
 #include "../lib/shash.h"
+#include "../lib/util.h"
 
 static void
 parse_elp_list(cfg_t *cfg, shash_t *ht)
@@ -192,7 +193,7 @@ parse_lcafs(cfg_t *cfg)
 
 
 int
-parse_mapping_cfg_params(cfg_t *map, conf_mapping_t *conf_mapping, uint8_t is_local)
+parse_mapping_cfg_params(cfg_t *map, conf_mapping_t *conf_mapping, uint8_t is_local, uint8_t is_static)
 {
 
     int ctr;
@@ -205,8 +206,11 @@ parse_mapping_cfg_params(cfg_t *map, conf_mapping_t *conf_mapping, uint8_t is_lo
         return (BAD);
     }
 
-    strcpy(conf_mapping->eid_prefix,cfg_getstr(map, "eid-prefix"));
+    conf_mapping->eid_prefix = strdup(cfg_getstr(map, "eid-prefix"));
     conf_mapping->iid = cfg_getint(map, "iid");
+    if (!is_static) {
+        conf_mapping->ttl = cfg_getint(map, "ttl");
+    }
 
     for (ctr = 0; ctr < cfg_size(map, "rloc-address"); ctr++){
         rl = cfg_getnsec(map, "rloc-address", ctr);
@@ -228,14 +232,6 @@ parse_mapping_cfg_params(cfg_t *map, conf_mapping_t *conf_mapping, uint8_t is_lo
         for (ctr = 0; ctr < cfg_size(map, "rloc-iface"); ctr++){
             rl = cfg_getnsec(map, "rloc-iface", ctr);
             afi = cfg_getint(rl, "ip_version");
-            if (afi == 4){
-                afi = AF_INET;
-            }else if (afi == 6){
-                afi = AF_INET6;
-            }else{
-                OOR_LOG(LERR,"Configuration file: The conf_loc_iface->ip_version of the locator should be 4 (IPv4) or 6 (IPv6)");
-                return (BAD);
-            }
             if (cfg_getstr(rl, "interface") == NULL){
                 OOR_LOG(LWRN, "Configuration file: Mapping %s with no RLOC interface selected",conf_mapping->eid_prefix);
                 return (BAD);
@@ -256,14 +252,14 @@ parse_mapping_cfg_params(cfg_t *map, conf_mapping_t *conf_mapping, uint8_t is_lo
 
 mapping_t *
 parse_mapping(cfg_t *map, oor_ctrl_dev_t *dev, shash_t * lcaf_ht,
-        uint8_t is_local)
+        uint8_t is_local, uint8_t is_static)
 {
     mapping_t *mapping;
     conf_mapping_t *conf_mapping;
 
     conf_mapping = conf_mapping_new();
 
-    if (parse_mapping_cfg_params(map, conf_mapping, is_local) != GOOD){
+    if (parse_mapping_cfg_params(map, conf_mapping, is_local, is_static) != GOOD){
         return (NULL);
     }
     mapping = process_mapping_config(dev, lcaf_ht, conf_mapping, is_local);
@@ -298,6 +294,10 @@ int
 parse_proxy_etrs(cfg_t *cfg, lisp_xtr_t *xtr)
 {
     int n,i;
+    mcache_entry_t *ipv4_petrs_mc,*ipv6_petrs_mc;
+
+    ipv4_petrs_mc = mcache_get_all_space_entry(xtr->tr.map_cache,AF_INET);
+    ipv6_petrs_mc = mcache_get_all_space_entry(xtr->tr.map_cache,AF_INET6);
     /* PROXY-ETR CONFIG */
     n = cfg_size(cfg, "proxy-etr-ipv4");
     for(i = 0; i < n; i++) {
@@ -307,7 +307,7 @@ parse_proxy_etrs(cfg_t *cfg, lisp_xtr_t *xtr)
             return (BAD);
         }
 
-        if (add_proxy_etr_entry(xtr->petrs_ipv4,
+        if (add_proxy_etr_entry(ipv4_petrs_mc,
                 cfg_getstr(petr, "address"),
                 cfg_getint(petr, "priority"),
                 cfg_getint(petr, "weight")) == GOOD) {
@@ -325,7 +325,7 @@ parse_proxy_etrs(cfg_t *cfg, lisp_xtr_t *xtr)
             return (BAD);
         }
 
-        if (add_proxy_etr_entry(xtr->petrs_ipv6,
+        if (add_proxy_etr_entry(ipv6_petrs_mc,
                 cfg_getstr(petr, "address"),
                 cfg_getint(petr, "priority"),
                 cfg_getint(petr, "weight")) == GOOD) {
@@ -336,14 +336,12 @@ parse_proxy_etrs(cfg_t *cfg, lisp_xtr_t *xtr)
     }
 
     /* Calculate forwarding info for petrs */
-    if (xtr->fwd_policy->init_map_cache_policy_inf(xtr->fwd_policy_dev_parm,xtr->petrs_ipv4) != GOOD){
+    if (xtr->tr.fwd_policy->init_map_cache_policy_inf(xtr->tr.fwd_policy_dev_parm,ipv4_petrs_mc) != GOOD){
         OOR_LOG(LDBG_1, "parse_proxy_etrs: Couldn't initiate routing info for PeTRs!.");
-        mcache_entry_del(xtr->petrs_ipv4);
         return(BAD);
     }
-    if (xtr->fwd_policy->init_map_cache_policy_inf(xtr->fwd_policy_dev_parm,xtr->petrs_ipv6) != GOOD){
+    if (xtr->tr.fwd_policy->init_map_cache_policy_inf(xtr->tr.fwd_policy_dev_parm,ipv6_petrs_mc) != GOOD){
         OOR_LOG(LDBG_1, "parse_proxy_etrs: Couldn't initiate routing info for PeTRs!.");
-        mcache_entry_del(xtr->petrs_ipv6);
         return(BAD);
     }
     return (GOOD);
@@ -376,7 +374,7 @@ parse_database_mapping(cfg_t *cfg, lisp_xtr_t *xtr, shash_t *lcaf_ht)
 
     n = cfg_size(cfg, "database-mapping");
     for (i = 0; i < n; i++) {
-        mapping = parse_mapping(cfg_getnsec(cfg, "database-mapping", i),&(xtr->super),lcaf_ht,TRUE);
+        mapping = parse_mapping(cfg_getnsec(cfg, "database-mapping", i),&(xtr->super),lcaf_ht,TRUE,FALSE);
         if (mapping == NULL){
             return (BAD);
         }
@@ -385,8 +383,8 @@ parse_database_mapping(cfg_t *cfg, lisp_xtr_t *xtr, shash_t *lcaf_ht)
             mapping_del(mapping);
             continue;
         }
-        if (xtr->fwd_policy->init_map_loc_policy_inf(
-                xtr->fwd_policy_dev_parm,map_loc_e,NULL)!= GOOD){
+        if (xtr->tr.fwd_policy->init_map_loc_policy_inf(
+                xtr->tr.fwd_policy_dev_parm,map_loc_e,NULL)!= GOOD){
             OOR_LOG(LERR, "Couldn't inititate forward information for mapping with EID: %s. Discarding it...",
                     lisp_addr_to_char(mapping_eid(mapping)));
             map_local_entry_del(map_loc_e);
@@ -403,54 +401,58 @@ parse_database_mapping(cfg_t *cfg, lisp_xtr_t *xtr, shash_t *lcaf_ht)
 }
 
 int
-configure_tunnel_router(cfg_t *cfg, lisp_xtr_t *xtr, shash_t *lcaf_ht)
+configure_tunnel_router(cfg_t *cfg, oor_ctrl_dev_t *dev, lisp_tr_t *tr, shash_t *lcaf_ht)
 {
     int i,n,ret;
     char *map_resolver;
-    char *encap;
+    char *encap, *encap_str;
     mapping_t *mapping;
+    mcache_entry_t *mce;
 
     /* FWD POLICY STRUCTURES */
 #ifdef VPP
-    xtr->fwd_policy = fwd_policy_class_find("vpp_balancing");
+    tr->fwd_policy = fwd_policy_class_find("vpp_balancing");
 #else
-    xtr->fwd_policy = fwd_policy_class_find("flow_balancing");
+    tr->fwd_policy = fwd_policy_class_find("flow_balancing");
 #endif
-    xtr->fwd_policy_dev_parm = xtr->fwd_policy->new_dev_policy_inf(ctrl_dev,NULL);
+    tr->fwd_policy_dev_parm = tr->fwd_policy->new_dev_policy_inf(ctrl_dev,NULL);
 
-    if ((encap = cfg_getstr(cfg, "encapsulation")) != NULL) {
-        if (strcmp(encap, "LISP") == 0) {
-            xtr->encap_type = ENCP_LISP;
-        }else if (strcmp(encap, "VXLAN-GPE") == 0){
-            xtr->encap_type = ENCP_VXLAN_GPE;
+    if ((encap_str = cfg_getstr(cfg, "encapsulation")) != NULL) {
+        encap = str_to_lower_case(encap_str);
+        if (strcmp(encap, "lisp") == 0) {
+            tr->encap_type = ENCP_LISP;
+        }else if (strcmp(encap, "vxlan-gpe") == 0){
+            tr->encap_type = ENCP_VXLAN_GPE;
         }else{
             OOR_LOG(LERR, "Unknown encapsulation type: %s",encap);
+            free(encap);
             return (BAD);
         }
+        free(encap);
     }
 
     /* RETRIES */
     ret = cfg_getint(cfg, "map-request-retries");
-    xtr->map_request_retries = (ret != 0) ? ret : DEFAULT_MAP_REQUEST_RETRIES;
+    tr->map_request_retries = (ret != 0) ? ret : DEFAULT_MAP_REQUEST_RETRIES;
 
 
     /* RLOC PROBING CONFIG */
     cfg_t *dm = cfg_getnsec(cfg, "rloc-probing", 0);
     if (dm != NULL) {
-        xtr->probe_interval = cfg_getint(dm, "rloc-probe-interval");
-        xtr->probe_retries = cfg_getint(dm, "rloc-probe-retries");
-        xtr->probe_retries_interval = cfg_getint(dm,
+        tr->probe_interval = cfg_getint(dm, "rloc-probe-interval");
+        tr->probe_retries = cfg_getint(dm, "rloc-probe-retries");
+        tr->probe_retries_interval = cfg_getint(dm,
                 "rloc-probe-retries-interval");
 
-        validate_rloc_probing_parameters(&xtr->probe_interval,
-                &xtr->probe_retries, &xtr->probe_retries_interval);
+        validate_rloc_probing_parameters(&tr->probe_interval,
+                &tr->probe_retries, &tr->probe_retries_interval);
     } else {
         OOR_LOG(LDBG_1, "Configuration file: RLOC probing not defined. "
                 "Setting default values: RLOC Probing Interval: %d sec.",
                 RLOC_PROBING_INTERVAL);
-        xtr->probe_interval = RLOC_PROBING_INTERVAL;
-        xtr->probe_retries = DEFAULT_RLOC_PROBING_RETRIES;
-        xtr->probe_retries_interval = DEFAULT_RLOC_PROBING_RETRIES_INTERVAL;
+        tr->probe_interval = RLOC_PROBING_INTERVAL;
+        tr->probe_retries = DEFAULT_RLOC_PROBING_RETRIES;
+        tr->probe_retries_interval = DEFAULT_RLOC_PROBING_RETRIES_INTERVAL;
 
     }
 
@@ -459,7 +461,7 @@ configure_tunnel_router(cfg_t *cfg, lisp_xtr_t *xtr, shash_t *lcaf_ht)
     n = cfg_size(cfg, "map-resolver");
     for(i = 0; i < n; i++) {
         if ((map_resolver = cfg_getnstr(cfg, "map-resolver", i)) != NULL) {
-            if (add_server(map_resolver, xtr->map_resolvers) == GOOD){
+            if (add_server(map_resolver, tr->map_resolvers) == GOOD){
                 OOR_LOG(LDBG_1, "Added %s to map-resolver list", map_resolver);
             }else{
                 OOR_LOG(LCRIT,"Can't add %s Map Resolver.",map_resolver);
@@ -471,15 +473,17 @@ configure_tunnel_router(cfg_t *cfg, lisp_xtr_t *xtr, shash_t *lcaf_ht)
     n = cfg_size(cfg, "static-map-cache");
     for (i = 0; i < n; i++) {
         cfg_t *smc = cfg_getnsec(cfg, "static-map-cache", i);
-        mapping = parse_mapping(smc,&(xtr->super),lcaf_ht,FALSE);
+        mapping = parse_mapping(smc,dev,lcaf_ht,FALSE,TRUE);
 
         if (mapping == NULL){
             OOR_LOG(LERR, "Can't add static Map Cache entry with EID prefix %s. Discarded ...",
                     cfg_getstr(smc, "eid-prefix"));
             return(BAD);
         }
-        if (mcache_lookup_exact(xtr->map_cache, mapping_eid(mapping)) == NULL){
-            if (tr_mcache_add_static_mapping(xtr, mapping) == GOOD){
+        if (mcache_lookup_exact(tr->map_cache, mapping_eid(mapping)) == NULL){
+            mce = tr_mcache_add_mapping(tr, mapping, MCE_STATIC, ACTIVE);
+            if (mce){
+                tr_mcache_entry_program_timers(tr,mce);
                 OOR_LOG(LDBG_1, "Added static Map Cache entry with EID prefix %s in the database.",
                         lisp_addr_to_char(mapping_eid(mapping)));
             }else{
@@ -501,11 +505,9 @@ configure_tunnel_router(cfg_t *cfg, lisp_xtr_t *xtr, shash_t *lcaf_ht)
 int
 configure_rtr(cfg_t *cfg)
 {
-    lisp_xtr_t *xtr;
+    lisp_rtr_t *rtr;
     shash_t *lcaf_ht;
-    map_local_entry_t *map_loc_e;
-    mapping_t *mapping;
-    int n,i;
+    int i,n;
 
     /* CREATE AND CONFIGURE RTR (xTR in fact) */
     if (ctrl_dev_create(RTR_MODE, &ctrl_dev) != GOOD) {
@@ -515,8 +517,8 @@ configure_rtr(cfg_t *cfg)
 
     lcaf_ht = parse_lcafs(cfg);
 
-    xtr = CONTAINER_OF(ctrl_dev, lisp_xtr_t, super);
-    if (configure_tunnel_router(cfg, xtr, lcaf_ht)!=GOOD){
+    rtr = lisp_rtr_cast(ctrl_dev);
+    if (configure_tunnel_router(cfg,&(rtr->super), &rtr->tr, lcaf_ht)!=GOOD){
         return (BAD);
     }
 
@@ -535,7 +537,7 @@ configure_rtr(cfg_t *cfg)
                 return (BAD);
             }
 
-            if (add_rtr_iface(xtr,
+            if (add_rtr_iface(rtr,
                     cfg_getstr(ri, "iface"),
                     cfg_getint(ri, "ip_version"),
                     cfg_getint(ri, "priority"),
@@ -547,44 +549,27 @@ configure_rtr(cfg_t *cfg)
                         cfg_getstr(ri, "iface"));
             }
         }
+        if (!rtr->all_locs_map->fwd_policy_info) {
+            /* RTR has all the configured interfaces down */
+            OOR_LOG(LERR, "Configuration file: All the configured interfaces doesn't exist or are down");
+            if (rtr->tr.fwd_policy->init_map_loc_policy_inf(
+                    rtr->tr.fwd_policy_dev_parm,rtr->all_locs_map,NULL) != GOOD){
+                OOR_LOG(LERR, "Couldn't initiate forward information for rtr localtors.");
+                map_local_entry_del(rtr->all_locs_map);
+                return (BAD);
+            }
+        }
     }
-
-    /* RTR DATABASE MAPPINGS (like for instance replication lists) */
-    n = cfg_size(cfg, "rtr-database-mapping");
-    for (i = 0; i < n; i++) {
-        mapping = parse_mapping(cfg_getnsec(cfg, "rtr-database-mapping",i),&(xtr->super),lcaf_ht,TRUE);
-        if (mapping == NULL){
+    n = cfg_size(cfg, "rtr-ms-node");
+    for(i = 0; i < n; i++) {
+        cfg_t *rms = cfg_getnsec(cfg, "rtr-ms-node", i);
+        if (rtr_add_rtr_ms_node(rtr,
+                cfg_getstr(rms, "address"),
+                cfg_getstr(rms, "key"),
+                cfg_getstr(rms, "draft-version")) != GOOD){
             return (BAD);
         }
-        map_loc_e = map_local_entry_new_init(mapping);
-        if (map_loc_e == NULL){
-            mapping_del(mapping);
-            continue;
-        }
-
-        if (xtr->fwd_policy->init_map_loc_policy_inf(
-                xtr->fwd_policy_dev_parm,map_loc_e,NULL) != GOOD){
-            OOR_LOG(LERR, "Couldn't initiate forward information for rtr database mapping with EID: %s. Discarding it...",
-                    lisp_addr_to_char(mapping_eid(mapping)));
-            map_local_entry_del(map_loc_e);
-            continue;
-        }
-
-        if (add_local_db_map_local_entry(map_loc_e,xtr) != GOOD){
-            map_local_entry_del(map_loc_e);
-            continue;
-        }
-
-
-        if (add_local_db_map_local_entry(map_loc_e,xtr) != GOOD){
-            map_local_entry_del(map_loc_e);
-        }
     }
-
-    /* Deallocate PiTRs and PeTRs elements */
-    glist_destroy(xtr->pitrs);
-    xtr->pitrs = NULL;
-
     shash_destroy(lcaf_ht);
 
     return(GOOD);
@@ -604,11 +589,12 @@ configure_xtr(cfg_t *cfg)
 
     lcaf_ht = parse_lcafs(cfg);
 
-    xtr = CONTAINER_OF(ctrl_dev, lisp_xtr_t, super);
+    xtr = lisp_xtr_cast(ctrl_dev);
 
     xtr->nat_aware = cfg_getbool(cfg, "nat_traversal_support") ? TRUE:FALSE;
     if(xtr->nat_aware){
         if (nat_set_xTR_ID(xtr) != GOOD){
+            OOR_LOG(LERR,"Could not generate xTR-ID");
         	return (BAD);
         }
         nat_set_site_ID(xtr, 0);
@@ -616,7 +602,7 @@ configure_xtr(cfg_t *cfg)
         OOR_LOG(LDBG_1, "NAT support enabled. Set defaul RLOC to IPv4 family");
     }
 
-    if (configure_tunnel_router(cfg, xtr, lcaf_ht)!=GOOD){
+    if (configure_tunnel_router(cfg, &(xtr->super), &xtr->tr, lcaf_ht)!=GOOD){
         return (BAD);
     }
 
@@ -655,12 +641,12 @@ configure_mn(cfg_t *cfg)
 
     lcaf_ht = parse_lcafs(cfg);
 
-    xtr = CONTAINER_OF(ctrl_dev, lisp_xtr_t, super);
+    xtr = lisp_xtr_cast(ctrl_dev);
 
     xtr->nat_aware = cfg_getbool(cfg, "nat_traversal_support") ? TRUE:FALSE;
     if(xtr->nat_aware){
         if (nat_set_xTR_ID(xtr) != GOOD){
-        	printf("aaaaaaaaaaaaa\n");
+            OOR_LOG(LERR,"Could not generate xTR-ID");
         	return (BAD);
         }
         nat_set_site_ID(xtr, 0);
@@ -668,7 +654,7 @@ configure_mn(cfg_t *cfg)
         OOR_LOG(LDBG_1, "NAT support enabled. Set defaul RLOC to IPv4 family");
     }
 
-    if (configure_tunnel_router(cfg, xtr, lcaf_ht)!=GOOD){
+    if (configure_tunnel_router(cfg, &(xtr->super), &xtr->tr, lcaf_ht)!=GOOD){
         return (BAD);
     }
 
@@ -694,20 +680,22 @@ configure_mn(cfg_t *cfg)
 int
 configure_ms(cfg_t *cfg)
 {
-    char *iface_name;
+    char *iface_name, *rtr_id;
     iface_t *iface=NULL;
     lisp_site_prefix_t *site;
     shash_t *lcaf_ht;
-    int i;
+    int i,j,n, res;
     lisp_ms_t *ms;
     mapping_t *mapping;
+    glist_t *rtr_id_list;
+
 
     /* create and configure xtr */
     if (ctrl_dev_create(MS_MODE, &ctrl_dev) != GOOD) {
         OOR_LOG(LCRIT, "Failed to create MS. Aborting!");
         exit_cleanup();
     }
-    ms = CONTAINER_OF(ctrl_dev, lisp_ms_t, super);
+    ms = lisp_ms_cast(ctrl_dev);
 
 
     /* create lcaf hash table */
@@ -770,7 +758,7 @@ configure_ms(cfg_t *cfg)
     for (i = 0; i< cfg_size(cfg, "ms-static-registered-site"); i++ ) {
         cfg_t *mss = cfg_getnsec(cfg, "ms-static-registered-site", i);
 
-        mapping = parse_mapping(mss,&(ms->super),lcaf_ht,FALSE);
+        mapping = parse_mapping(mss,&(ms->super),lcaf_ht,FALSE,FALSE);
 
         if (mapping == NULL){
             OOR_LOG(LERR, "Can't create static register site for %s",
@@ -795,6 +783,42 @@ configure_ms(cfg_t *cfg)
         }
     }
 
+    /* NAT RTR configuration of the MS */
+    for (i = 0; i< cfg_size(cfg, "ms-rtr-node"); i++ ) {
+        cfg_t *rtr_cfg = cfg_getnsec(cfg, "ms-rtr-node", i);
+        res = ms_add_rtr_node(ms,
+                cfg_getstr(rtr_cfg, "name"),
+                cfg_getstr(rtr_cfg, "address"),
+                cfg_getstr(rtr_cfg, "key"));
+        if (res != GOOD){
+            return(BAD);
+        }
+    }
+
+    for (i = 0; i< cfg_size(cfg, "ms-rtrs-set"); i++ ) {
+        cfg_t *rtr_set_cfg = cfg_getnsec(cfg, "ms-rtrs-set", i);
+        rtr_id_list = glist_new();
+        n = cfg_size(rtr_set_cfg, "rtrs");
+        for(j = 0; j < n; j++) {
+            if ((rtr_id = cfg_getnstr(rtr_set_cfg, "rtrs", j)) != NULL) {
+                glist_add(rtr_id,rtr_id_list);
+            }
+        }
+        res = ms_add_rtr_set(ms,
+                cfg_getstr(rtr_set_cfg, "name"),
+                cfg_getint(rtr_set_cfg, "ttl"),
+                rtr_id_list);
+        if (res != GOOD){
+            glist_destroy(rtr_id_list);
+            return(BAD);
+        }
+        glist_destroy(rtr_id_list);
+    }
+
+    if (ms_advertised_rtr_set(ms, cfg_getstr(cfg, "ms-advertised-rtrs-set")) != GOOD){
+        return (BAD);
+    }
+
     /* destroy the hash table */
     shash_destroy(lcaf_ht);
     return(GOOD);
@@ -805,8 +829,9 @@ handle_config_file()
 {
     int ret;
     cfg_t *cfg;
-    char *mode;
+    char *mode, *mode_str;
     char *log_file;
+    char *scope, *scope_str;
 
     /* xTR specific */
     static cfg_opt_t map_server_opts[] = {
@@ -835,6 +860,7 @@ handle_config_file()
     static cfg_opt_t db_mapping_opts[] = {
             CFG_STR("eid-prefix",           0, CFGF_NONE),
             CFG_INT("iid",                  0, CFGF_NONE),
+            CFG_INT("ttl",DEFAULT_DATA_CACHE_TTL, CFGF_NONE),
             CFG_SEC("rloc-address",         rloc_address_opts, CFGF_MULTI),
             CFG_SEC("rloc-iface",           rloc_iface_opts, CFGF_MULTI),
             CFG_END()
@@ -922,6 +948,28 @@ handle_config_file()
             CFG_END()
     };
 
+    static cfg_opt_t rtr_opts[] = {
+            CFG_STR("name",                        0, CFGF_NONE),
+            CFG_STR("address",                     0, CFGF_NONE),
+            CFG_STR("key",                         0, CFGF_NONE),
+            CFG_END()
+    };
+
+    static cfg_opt_t rtr_set_opts[] = {
+            CFG_STR("name",                     0, CFGF_NONE),
+            CFG_INT("ttl",         OOR_MS_RTR_TTL, CFGF_NONE),
+            CFG_STR_LIST("rtrs",                0, CFGF_NONE),
+            CFG_END()
+    };
+
+    static cfg_opt_t rtr_ms_opts[] = {
+            CFG_STR("address",                     0, CFGF_NONE),
+            CFG_STR("key",                         0, CFGF_NONE),
+            CFG_STR("draft-version","OLD",CFGF_NONE),
+            CFG_END()
+    };
+
+
     cfg_opt_t opts[] = {
             CFG_SEC("database-mapping",     db_mapping_opts,        CFGF_MULTI),
             CFG_SEC("ms-static-registered-site", db_mapping_opts, CFGF_MULTI),
@@ -937,6 +985,7 @@ handle_config_file()
             CFG_INT("control-port",         0, CFGF_NONE),
             CFG_INT("debug",                0, CFGF_NONE),
             CFG_STR("log-file",             0, CFGF_NONE),
+            CFG_STR("ipv6-scope",          "GLOBAL",               CFGF_NONE),
             CFG_INT("rloc-probing-interval",0, CFGF_NONE),
             CFG_STR_LIST("map-resolver",    0, CFGF_NONE),
             CFG_STR_LIST("proxy-itrs",      0, CFGF_NONE),
@@ -953,6 +1002,10 @@ handle_config_file()
             CFG_SEC("explicit-locator-path", elp_opts,              CFGF_MULTI),
             CFG_SEC("replication-list",     rle_opts,               CFGF_MULTI),
             CFG_SEC("multicast-info",       mc_info_opts,           CFGF_MULTI),
+            CFG_SEC("ms-rtrs-set",              rtr_set_opts,          CFGF_MULTI),
+            CFG_SEC("ms-rtr-node",              rtr_opts,              CFGF_MULTI),
+            CFG_STR("ms-advertised-rtrs-set",       0, CFGF_NONE),
+            CFG_SEC("rtr-ms-node",rtr_ms_opts,CFGF_MULTI),
             CFG_END()
     };
 
@@ -1008,21 +1061,41 @@ handle_config_file()
     if (daemonize == TRUE){
         open_log_file(log_file);
     }
-    mode = cfg_getstr(cfg, "operating-mode");
-    if (mode) {
-        if (strcmp(mode, "xTR") == 0) {
+
+    scope_str = cfg_getstr(cfg, "ipv6-scope");
+    scope = str_to_lower_case(scope_str);
+    if (strcmp(scope,"global") == 0){
+        ipv6_scope = SCOPE_GLOBAL;
+        OOR_LOG (LDBG_1, "Selected IPv6 scope: Global");
+    }else if (strcmp(scope,"site") == 0){
+        ipv6_scope = SCOPE_SITE_LOCAL;
+        OOR_LOG (LDBG_1, "Selected IPv6 scope: Site local");
+    }else{
+        OOR_LOG (LCRIT, "Configuration file: Unknown IPv6 scope: %s",scope_str);
+        free(scope);
+        return (BAD);
+    }
+    free(scope);
+
+
+    mode_str = cfg_getstr(cfg, "operating-mode");
+    if (mode_str) {
+        mode = str_to_lower_case(mode_str);
+        if (strcmp(mode, "xtr") == 0) {
             ret=configure_xtr(cfg);
-        } else if (strcmp(mode, "MS") == 0) {
+        } else if (strcmp(mode, "ms") == 0) {
             ret=configure_ms(cfg);
-        } else if (strcmp(mode, "RTR") == 0) {
+        } else if (strcmp(mode, "rtr") == 0) {
             ret=configure_rtr(cfg);
-        }else if (strcmp(mode, "MN") == 0) {
+        }else if (strcmp(mode, "mn") == 0) {
             ret=configure_mn(cfg);
         }else{
             OOR_LOG (LCRIT, "Configuration file: Unknown operating mode: %s",mode);
             cfg_free(cfg);
+            free(mode);
             return (BAD);
         }
+        free(mode);
     }
 
     cfg_free(cfg);
