@@ -68,6 +68,7 @@ typedef struct htable_nonces_ htable_nonces_t;
 #define OOR_MAX_PROBE_RETRANSMIT      1  // Maximum amount of RLOC probe MRq retransmissions
 #define OOR_MAX_RETRANSMITS           5  // Maximum amount of retransmits of a message
 #define OOR_MIN_RETRANSMIT_INTERVAL   1  // Minimum time between retransmits of control messages
+#define OOR_MS_RTR_TTL                1440 // TTL in minutes of a RTR list learned by an xTR
 
 
 #define DEFAULT_MAP_REQUEST_RETRIES             3
@@ -90,7 +91,7 @@ typedef struct htable_nonces_ htable_nonces_t;
  */
 
 #define EVER            ;;
-#define OOR_VERSION   "v1.2.0"
+#define OOR_VERSION   "v1.2.1"
 #define OOR           "oor"
 #define PID_FILE      "/var/run/oor.pid"
 #define DEVICE_ID_FILE "oor-id"
@@ -103,6 +104,7 @@ typedef struct htable_nonces_ htable_nonces_t;
 #define PACKED          __attribute__ ((__packed__))
 #define uchar           u_char
 
+#define UPDATED             2
 #define GOOD                1
 #define BAD                 0
 #define ERR_SRC_ADDR        -1
@@ -140,14 +142,16 @@ typedef struct htable_nonces_ htable_nonces_t;
 #define FULL_NAT            2
 
 
-/*
- * Structure to simplify netlink processing
- */
-typedef struct nlsock_handle
-{
-    int         fd;       // netlink socket fd
-    uint32_t    seq;      // netlink message seq number
-} nlsock_handle;
+typedef enum {
+    NAT_PREV_DRAFT_4,
+    NAT_AFTER_DRAFT_4,
+}nat_version;
+
+typedef enum {
+    SCOPE_GLOBAL,
+    SCOPE_SITE_LOCAL,
+    SCOPE_LINK_LOCAL
+} ipv6_scope_e;
 
 
 
