@@ -35,29 +35,27 @@ lisp_msg_type(lbuf_t *b)
     return(hdr->type);
 }
 
-static void *
+void *
 lisp_msg_pull_ecm_hdr(lbuf_t *b)
 {
     lbuf_reset_lisp_hdr(b);
     return(lbuf_pull(b, sizeof(ecm_hdr_t)));
 }
 
-/* Process encapsulated map request header:  lisp header and the interal IP and
- * UDP header */
 int
-lisp_msg_ecm_decap(lbuf_t *pkt, uint16_t *src_port)
+lisp_msg_parse_int_ip_udp(lbuf_t *pkt)
 {
     uint16_t ipsum = 0;
     uint16_t udpsum = 0;
     int udp_len = 0;
     struct udphdr *udph;
     struct ip *iph;
-
-    /* this is the new start of the packet */
-    lisp_msg_pull_ecm_hdr(pkt);
     /* Set and extract inner layer 3 packet */
     lbuf_reset_l3(pkt);
     iph = pkt_pull_ip(pkt);
+    if (!iph){
+        return (BAD);
+    }
     /* Set and extract inner layer 4 packet */
     lbuf_reset_l4(pkt);
     udph = pkt_pull_udp(pkt);
@@ -65,9 +63,6 @@ lisp_msg_ecm_decap(lbuf_t *pkt, uint16_t *src_port)
     /* Set the beginning of the LISP msg*/
     lbuf_reset_lisp(pkt);
 
-    /* This should overwrite the external port (dst_port in map-reply =
-     * inner src_port in encap map-request) */
-    *src_port = ntohs(udpsport(udph));
     udp_len = ntohs(udplen(udph));
 
     /* Verify the checksums. */
@@ -90,8 +85,7 @@ lisp_msg_ecm_decap(lbuf_t *pkt, uint16_t *src_port)
         }
     }
 
-    OOR_LOG(LDBG_2, "%s, inner IP: %s -> %s, inner UDP: %d -> %d",
-            lisp_msg_hdr_to_char(pkt),
+    OOR_LOG(LDBG_2, "Inner IP: %s -> %s, inner UDP: %d -> %d",
             ip_to_char(&iph->ip_src, ip_version_to_sock_afi(iph->ip_v)),
             ip_to_char(&iph->ip_dst, ip_version_to_sock_afi(iph->ip_v)),
             ntohs(udpsport(udph)), ntohs(udpdport(udph)));
@@ -259,6 +253,30 @@ err:
     return(BAD);
 }
 
+int
+lisp_msg_parse_inf_req_eid_ttl(lbuf_t *b, lisp_addr_t *eid, int *ttl)
+{
+    *ttl = (ntohl(*(int *)lbuf_data(b)));
+    lbuf_pull(b,sizeof(int));
+
+    if (lisp_msg_parse_eid_rec(b, eid)!= GOOD){
+        return(BAD);
+    }
+    return(GOOD);
+}
+
+int
+lisp_msg_parse_xtr_id_site_id (lbuf_t *b, lisp_xtr_id *xtr_id, lisp_site_id *site_id)
+{
+    void *ptr;
+    ptr = lbuf_pull(b, sizeof(lisp_xtr_id));
+    memcpy(xtr_id,ptr,sizeof(lisp_xtr_id));
+    ptr = lbuf_pull(b, sizeof(lisp_site_id));
+    memcpy(site_id,ptr,sizeof(lisp_site_id));
+    return(GOOD);
+}
+
+
 static unsigned int
 msg_type_to_hdr_len(lisp_msg_type_e type)
 {
@@ -286,19 +304,6 @@ lisp_msg_pull_hdr(lbuf_t *b)
 }
 
 
-void *
-lisp_msg_pull_auth_field(lbuf_t *b)
-{
-    void *hdr;
-    lisp_key_type_e keyid;
-
-    hdr = lbuf_pull(b, sizeof(auth_record_hdr_t));
-    keyid = ntohs(AUTH_REC_KEY_ID(hdr));
-    lbuf_pull(b, auth_data_get_len_for_type(keyid));
-    return(hdr);
-}
-
-
 void *
 lisp_msg_put_addr(lbuf_t *b, lisp_addr_t *addr)
 {
@@ -334,7 +339,6 @@ lisp_msg_put_locator(lbuf_t *b, locator_t *locator)
     return(loc_ptr);
 }
 
-
 static void
 increment_record_count(lbuf_t *b)
 {
@@ -391,9 +395,6 @@ lisp_msg_put_mapping(
 
     /* Add locators */
     mapping_foreach_active_locator(m,loct){
-        if (locator_state(loct) == DOWN){
-            continue;
-        }
         ploc = lisp_msg_put_locator(b, loct);
         if (probed_loc)
             if (probed_loc != NULL
@@ -465,11 +466,18 @@ lisp_msg_put_eid_rec(lbuf_t *b, lisp_addr_t *eid)
     return(hdr);
 }
 
+void *
+lisp_msg_put_xtr_id_site_id(lbuf_t *b, lisp_xtr_id *xTR_id, lisp_site_id *site_id)
+{
+    void * ptr;
+    ptr = lbuf_put(b, xTR_id, sizeof(lisp_xtr_id));
+    lbuf_put(b, site_id, sizeof(lisp_site_id));
+    return (ptr);
+}
+
 void *
 lisp_msg_encap(lbuf_t *b, int lp, int rp, lisp_addr_t *la, lisp_addr_t *ra)
 {
-    void *hdr;
-
     /* end of lisp msg */
     lbuf_reset_lisp(b);
     lisp_addr_t *ip_la, *ip_ra;
@@ -484,14 +492,10 @@ lisp_msg_encap(lbuf_t *b, int lp, int rp, lisp_addr_t *la, lisp_addr_t *ra)
         ip_ra = lisp_addr_get_ip_pref_addr(ra);
     }
 
-
     /* push inner ip and udp */
-    pkt_push_udp_and_ip(b, lp, rp,lisp_addr_ip(ip_la),lisp_addr_ip(ip_ra));
+    pkt_push_inner_udp_and_ip(b, lp, rp,lisp_addr_ip(ip_la),lisp_addr_ip(ip_ra));
 
-    /* push lisp ecm hdr */
-    hdr = lbuf_push_uninit(b, sizeof(ecm_hdr_t));
-    ecm_hdr_init(hdr);
-    lbuf_reset_lisp_hdr(b);
+    lisp_msg_push_encap_lisp_header(b);
 
     OOR_LOG(LDBG_1, "%s, inner IP: %s -> %s, inner UDP: %d -> %d",
                 lisp_msg_ecm_hdr_to_char(b), lisp_addr_to_char(la),
@@ -501,6 +505,18 @@ lisp_msg_encap(lbuf_t *b, int lp, int rp, lisp_addr_t *la, lisp_addr_t *ra)
     return(lbuf_data(b));
 }
 
+void *
+lisp_msg_push_encap_lisp_header(lbuf_t *b)
+{
+    void *hdr;
+    /* push lisp ecm hdr */
+    hdr = lbuf_push_uninit(b, sizeof(ecm_hdr_t));
+    ecm_hdr_init(hdr);
+    lbuf_reset_lisp_hdr(b);
+
+    return(lbuf_data(b));
+}
+
 lbuf_t *
 lisp_msg_create_buf()
 {
@@ -594,56 +610,70 @@ lisp_msg_inf_req_create(mapping_t *m, lisp_key_type_e keyid)
     lisp_addr_t addr;
 
     if (!lisp_msg_put_empty_auth_record(b, keyid)) {
+        lbuf_del(b);
         return(NULL);
     }
 
     if (!lisp_msg_put_inf_req_hdr_2(b, mapping_eid(m), 0)) {
+        lbuf_del(b);
         return(NULL);
     }
 
     lisp_addr_set_lafi(&addr, LM_AFI_NO_ADDR);
     if (lisp_msg_put_addr(b, &addr) == NULL) {
+        lbuf_del(b);
         return(NULL);
     }
 
     return(b);
 }
 
+lbuf_t *
+lisp_msg_inf_reply_create(lisp_addr_t *eid, lisp_addr_t *nat_lcaf, lisp_key_type_e keyid, int ttl)
+{
+    lbuf_t *b = lisp_msg_create(LISP_INFO_NAT);
+    void * hdr;
+
+
+    if (!lisp_msg_put_empty_auth_record(b, keyid)) {
+        lbuf_del(b);
+        return(NULL);
+    }
+
+    if (!lisp_msg_put_inf_req_hdr_2(b, eid, ttl)) {
+        lbuf_del(b);
+        return(NULL);
+    }
+
+    if (lisp_msg_put_addr(b, nat_lcaf) == NULL) {
+        lbuf_del(b);
+        return(NULL);
+    }
+
+    hdr = lisp_msg_hdr(b);
+    INF_REQ_R_bit(hdr)= 1;
+
+    return(b);
+}
+
 lbuf_t *
 lisp_msg_mreg_create(mapping_t *m, lisp_key_type_e keyid)
 {
     lbuf_t *b = lisp_msg_create(LISP_MAP_REGISTER);
 
     if (!lisp_msg_put_empty_auth_record(b, keyid)) {
+        lbuf_del(b);
         return(NULL);
     }
 
     if (!lisp_msg_put_mapping(b, m, NULL)) {
+        lbuf_del(b);
         return(NULL);
     }
 
     return(b);
 }
 
-lbuf_t *
-lisp_msg_nat_mreg_create(mapping_t *m,lisp_site_id site_id,
-        lisp_xtr_id *xtr_id, lisp_key_type_e keyid)
-{
-    lbuf_t *b = lisp_msg_create(LISP_MAP_REGISTER);
-    if (!lisp_msg_put_empty_auth_record(b, keyid)){
-        return(NULL);
-    }
-
-    if (!lisp_msg_put_mapping(b, m, NULL)) {
-        return(NULL);
-    }
-
-    lbuf_put(b, xtr_id, sizeof(lisp_xtr_id));
-    lbuf_put(b, &site_id, sizeof(lisp_site_id));
-
-    return(b);
-}
-
 char *
 lisp_msg_hdr_to_char(lbuf_t *b)
 {
@@ -683,42 +713,96 @@ lisp_msg_ecm_hdr_to_char(lbuf_t *b)
     return(ecm_hdr_to_char(h));
 }
 
-int
-lisp_msg_fill_auth_data(lbuf_t *b, lisp_key_type_e keyid, const char *key)
-{
-    void *hdr = lisp_msg_auth_record(b);
+/************************ ECM Auth header ************************************/
 
+ecm_auth_data_type
+lisp_ecm_auth_type(lbuf_t *b)
+{
+    ecm_auth_field_hdr *auth_hdr;
+    ecm_hdr_t *hdr = lbuf_lisp_hdr(b);
+    if (ECM_SECURITY_BIT(hdr) == 0){
+        return (NO_AUTH_DATA);
+    }
+    auth_hdr = (ecm_auth_field_hdr *)((uint8_t *)hdr + sizeof(ecm_hdr_t));
+    return(auth_hdr->ad_type);
+}
+
+int
+lisp_msg_fill_rtr_auth_data(lbuf_t *b, void *rtr_auth_hdr, lisp_key_type_e keyid, const char *key)
+{
+    void *auth_record_hdr = RTR_AUTH_REC(rtr_auth_hdr);
+    lbuf_t buff = *b;
+    /* The authentication process is of the Map Notify data */
+    lbuf_point_to_lisp(&buff);
+    return (lisp_msg_fill_auth_data(&buff, auth_record_hdr, keyid, key));
+}
+
+int
+lisp_msg_check_rtr_auth_data(lbuf_t *b, void *rtr_auth_hdr, const char *key)
+{
+    void *auth_record_hdr = RTR_AUTH_REC(rtr_auth_hdr);
+    lbuf_t buff = *b;
+    lbuf_point_to_lisp(&buff);
+    return(lisp_msg_check_auth_field(&buff,auth_record_hdr,key));
+}
+
+void *
+lisp_msg_push_empty_rtr_auth_data(lbuf_t *b, lisp_key_type_e keyid)
+{
+    void *hdr;
+    uint16_t len = auth_data_get_len_for_type(keyid);
+    hdr = lbuf_push_uninit(b, sizeof(rtr_auth_field_hdr) + len);
+    memset(hdr, 0, sizeof(rtr_auth_field_hdr) + len);
+    ECM_AUTH_TYPE(hdr) = RTR_AUTH_DATA;
+    RTR_AUTH_KEY_ID(hdr) = htons(keyid);
+    RTR_AUTH_DATA_LEN(hdr) = htons(len);
+    return(hdr);
+}
+
+void *
+lisp_msg_pull_rtr_auth_field(lbuf_t *b)
+{
+    void *hdr;
+    lisp_key_type_e keyid;
+
+    hdr = lbuf_pull(b, sizeof(rtr_auth_field_hdr));
+    keyid = ntohs(RTR_AUTH_KEY_ID(hdr));
+    lbuf_pull(b, auth_data_get_len_for_type(keyid));
+    return(hdr);
+}
+
+/*************************** Auth Record *************************************/
+
+int
+lisp_msg_fill_auth_data(lbuf_t *b, void *auth_record_hdr, lisp_key_type_e keyid, const char *key)
+{
+    AUTH_REC_KEY_ID(auth_record_hdr) = htons(keyid);
+    AUTH_REC_DATA_LEN(auth_record_hdr) = htons(auth_data_get_len_for_type(keyid));
     if (complete_auth_fields(
             keyid,
             key,
             lbuf_lisp(b),
             lbuf_size(b),
-            AUTH_REC_DATA(hdr)) != GOOD) {
+            AUTH_REC_DATA(auth_record_hdr)) != GOOD) {
         return(BAD);
     }
 
     return(GOOD);
 }
 
-
-
 /* Checks auth field of Map-Register, Map-Notify and Info-Reply messages */
 int
-lisp_msg_check_auth_field(lbuf_t *b, const char *key)
+lisp_msg_check_auth_field(lbuf_t *b, void *auth_record_hdr, const char *key)
 {
     lisp_key_type_e keyid;
     uint16_t        ad_len  = 0;
     int             ret     = BAD;
 
-    auth_record_hdr_t *hdr;
-
-    hdr = lisp_msg_auth_record(b);
-
-    keyid = ntohs(AUTH_REC_KEY_ID(hdr));
+    keyid = ntohs(AUTH_REC_KEY_ID(auth_record_hdr));
     ad_len = auth_data_get_len_for_type(keyid);
-    if (ad_len != ntohs(AUTH_REC_DATA_LEN(hdr))) {
+    if (ad_len != ntohs(AUTH_REC_DATA_LEN(auth_record_hdr))) {
         OOR_LOG(LDBG_3, "Auth Record record length is wrong: %d instead of %d",
-                ntohs(AUTH_REC_DATA_LEN(hdr)), ad_len);
+                ntohs(AUTH_REC_DATA_LEN(auth_record_hdr)), ad_len);
         return(BAD);
     }
 
@@ -727,7 +811,10 @@ lisp_msg_check_auth_field(lbuf_t *b, const char *key)
             key,
             lbuf_lisp(b),
             lbuf_size(b),
-            AUTH_REC_DATA(hdr));
+            AUTH_REC_DATA(auth_record_hdr));
+    if (ret != GOOD){
+        OOR_LOG(LDBG_3, "Wrong authentication data");
+    }
 
     return(ret);
 }
@@ -744,17 +831,42 @@ lisp_msg_put_empty_auth_record(lbuf_t *b, lisp_key_type_e keyid)
     return(hdr);
 }
 
+void *
+lisp_msg_push_empty_auth_record(lbuf_t *b, lisp_key_type_e keyid)
+{
+    void *hdr;
+    uint16_t len = auth_data_get_len_for_type(keyid);
+    hdr = lbuf_push_uninit(b, sizeof(auth_record_hdr_t) + len);
+    AUTH_REC_KEY_ID(hdr) = htons(keyid);
+    AUTH_REC_DATA_LEN(hdr) = htons(len);
+    memset(AUTH_REC_DATA(hdr), 0, len);
+    return(hdr);
+}
+
+void *
+lisp_msg_pull_auth_field(lbuf_t *b)
+{
+    void *hdr;
+    lisp_key_type_e keyid;
+
+    hdr = lbuf_pull(b, sizeof(auth_record_hdr_t));
+    keyid = ntohs(AUTH_REC_KEY_ID(hdr));
+    lbuf_pull(b, auth_data_get_len_for_type(keyid));
+    return(hdr);
+}
+
+/*****************************************************************************/
 void *
 lisp_msg_put_inf_req_hdr_2(lbuf_t *b, lisp_addr_t *eid_pref, uint8_t ttl)
 {
     void *hdr;
     lisp_addr_t *eid;
-    hdr = lbuf_put_uninit(b, sizeof(info_nat_hdr_2_t));
 
-    INF_REQ_2_TTL(hdr) = ttl;
+    hdr = lbuf_put_uninit(b, sizeof(info_nat_hdr_2_t));
+    INF_REQ_2_TTL(hdr) = htonl(ttl);
     eid = lisp_addr_get_ip_pref_addr(eid_pref);
     INF_REQ_2_EID_MASK(hdr) = lisp_addr_ip_get_plen(eid);
-    if (lisp_msg_put_addr(b, eid) == NULL) {
+    if (lisp_msg_put_addr(b, eid_pref) == NULL) {
         return(NULL);
     }
 
@@ -798,35 +910,38 @@ lisp_data_pull_hdr(lbuf_t *b)
     return(dt);
 }
 
-
-
-/* returns in 'addr_' the first element of the list 'l' to have AFI 'afi'
- * caller must allocate and free 'addr_' */
 int
-laddr_list_get_addr(glist_t *l, int afi, lisp_addr_t *addr)
+laddr_list_has_addr(glist_t *l, lisp_addr_t *addr)
 {
     glist_entry_t *it;
-    lisp_addr_t *ait;
-    int found = 0;
+    lisp_addr_t *aux_addr;
 
     if (!addr) {
-        return(BAD);
+        return(FALSE);
     }
-
     glist_for_each_entry(it, l) {
-        ait = (lisp_addr_t *)glist_entry_data(it);
-        if (lisp_addr_ip_afi(ait) == afi) {
-            lisp_addr_copy(addr, ait);
-            found = 1;
-            break;
+        aux_addr = (lisp_addr_t *)glist_entry_data(it);
+        if (lisp_addr_cmp(addr,aux_addr)==0){
+            return(TRUE);
         }
     }
+    return(FALSE);
+}
 
-    if (found) {
-        return(GOOD);
-    } else {
-        return(BAD);
+/* returns ptr to the first element of the list 'l' to have AFI 'afi'*/
+lisp_addr_t *
+laddr_list_get_fst_addr_with_afi(glist_t *l, int afi)
+{
+    glist_entry_t *it;
+    lisp_addr_t *addr;
+
+    glist_for_each_entry(it, l) {
+        addr = (lisp_addr_t *)glist_entry_data(it);
+        if (lisp_addr_ip_afi(addr) == afi) {
+            return (addr);
+        }
     }
+    return (NULL);
 }
 
 char *
