@@ -72,14 +72,14 @@ typedef enum lisp_msg_type_ {
  * header of the encapsulated LISP control message.
  *
  *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
- *    |Type=8 |S|D|R|             Reserved                            |
+ *    |Type=8 |S|D|E|M|-|R|N|        Reserved                         |
  *    +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  */
 
 typedef struct ecm_hdr {
 #ifdef LITTLE_ENDIANS
-    uint8_t n_bit:1; // RTR relays the ECM-ed Map-Register to a Map-Server
-    uint8_t r_bit:1; // The encapsulated msg is to be processed by an RTR
+    uint8_t m_bit:1;
+    uint8_t e_bit:1;
     uint8_t d_bit:1; // DDT originated
     uint8_t s_bit:1;
     uint8_t type:4;
@@ -87,10 +87,21 @@ typedef struct ecm_hdr {
     uint8_t type:4;
     uint8_t s_bit:1;
     uint8_t d_bit:1;
+    uint8_t e_bit:1;
+    uint8_t m_bit:1;
+#endif
+#ifdef LITTLE_ENDIANS
+    uint8_t reserved:5;
+    uint8_t n_bit:1; // RTR relays the ECM-ed Map-Register to a Map-Server
+    uint8_t r_bit:1; // The encapsulated msg is to be processed by an RTR
+    uint8_t not_used_bit:1;
+#else
+    uint8_t not_used_bit:1;
     uint8_t r_bit:1;
     uint8_t n_bit:1;
+    uint8_t reserved:5;
 #endif
-    uint8_t reserved2[3];
+    uint8_t reserved2[2];
 } ecm_hdr_t;
 
 
@@ -102,6 +113,7 @@ void ecm_hdr_init(void *ptr);
 #define ECM_SECURITY_BIT(h_) (ECM_HDR_CAST(h_))->s_bit
 #define ECM_DDT_BIT(h_) (ECM_HDR_CAST(h_))->d_bit
 #define ECM_RTR_PROCESS_BIT(h_) (ECM_HDR_CAST(h_))->r_bit
+#define ECM_RTR_RELAYED_BIT(h_) (ECM_HDR_CAST(h_))->n_bit
 
 
 /*
@@ -192,20 +204,10 @@ char *map_request_hdr_to_char(map_request_hdr_t *h);
 #define MREQ_SMR_INVOKED(h_) (MREQ_HDR_CAST(h_))->smr_invoked
 
 
-
-
-
-
 /*
  * MAP-REPLY MESSAGE
  */
 
- /*  Map Reply action codes */
- #define LISP_ACTION_NO_ACTION           0
- #define LISP_ACTION_FORWARD             1
- #define LISP_ACTION_DROP                2
- #define LISP_ACTION_SEND_MAP_REQUEST    3
-
  /*
   * Map-Reply Message Format
   *
@@ -398,7 +400,10 @@ char *map_notify_hdr_to_char(map_notify_hdr_t *h);
  *   +-> +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
  */
 
-/* I and R bit are defined on NAT tarversal draft*/
+/* I and R bit are defined on NAT tarversal draft
+ * R bit is defined in NAT draft v2 but it has been moved to Encap header
+ * As many cisco routers implements v2, we still matain this bit */
+
 
 typedef struct map_register_hdr {
 #ifdef LITTLE_ENDIANS
